/******************************************************************************
* File name:
*   MovieQuotes.java
* Author:
*   Barham, Ray
*
* Creation Date:
*   3/25/2007
* 
* Developed on:
*   WinXP
* Developed with:
*   Java(TM) SE Development Kit 6
*   JCreator 4.00.028
*
* Description:
*   This is a poor first attempt at working with XML.  Please be kind...
*   See below for the description of the class.
*   Required format for the XML file:
*
*     <moviequotes>
*     <movie title="Movie Title">
*     <quote>
*     This is a quote.
*     </quote>
*     <quote>
*     This is a second quote.
*     </quote>
*     </movie>
*     <movie title="Another movie Title">
*     <quote>
*     This is a quote.
*     </quote>
*     </movie>
*     </moviequotes>
*
*   Note: The tag names and movie attributes are required by the class.
*   Also, it is formated without whitespace on purpose.  I haven't figured
*   out how to remove whitespace yet.
*
* Referenced information:
*   Core Java 2 - Advanced Features, Volume 2, Horstmann & Cornell, Page 1109
******************************************************************************/

// Import statements
import java.io.*;
import java.lang.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;

/******************************************************************************
* Class Name:
*   MovieQuotes
* Base Class:
*   None
* Class Description:
*   This class is used to parse through an XML file of movies and quotes.
*   This class only allows parsing of one movie at a time.  In order specify 
*   a movie to parse, see the setMovie(...) method(s).
******************************************************************************/
public class MovieQuoteParser
{
	// NodeList objects used to access the XML data
	private NodeList m_MovieNL;
	private NodeList m_QuoteNL;
	
	private int m_MovieIndex;
	
	/**************************************************************************
	* Method Name:
	*   MovieQuoteParser
	* Arguments Passed:
	*   File XMLFile = The XML file to parse.  See above description for
	*     information on the format of the XML file.
	* Data Returned:
	*   None
	* Method Description:
	*   This is the default constructor to the MovieQuoteParser class.  It sets
	*   up the objects for accessing the XML file and extracts all the movie
	*   data.
	**************************************************************************/
	public MovieQuoteParser(File XMLFile)
		throws IOException, ParserConfigurationException, SAXException
	{
		m_MovieIndex = -1;
		
		// Objects required to implament XML support
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document doc = builder.parse(XMLFile);
		
		doc.getDocumentElement().normalize();
		
		m_MovieNL = doc.getElementsByTagName("movie");
	}
	
	/**************************************************************************
	* Method Name:
	*   getMovieTitles
	* Arguments Passed:
	*   None
	* Data Returned:
	*   String[] = An array containing the movie titles, in order of index
	* Method Description:
	*   This method returns a list of the movie titles from the XML file.  This
	*   list is in the order that they are parsed.
	**************************************************************************/
	public String[] getMovieTitles()
	{
		String movieTitles[] = new String[getNumberOfMovies()];
		
		if(movieTitles.length > 0)
		{
			for(int i = 0; i < movieTitles.length; i++)
				movieTitles[i] = ((Element)m_MovieNL.item(i)).getAttribute("title");
			
			return movieTitles;
		}
		
		return null;
	}
	
	/**************************************************************************
	* Method Name:
	*   getNumberOfMovies
	* Arguments Passed:
	*   None
	* Data Returned:
	*   int = the number of movies in the XML file
	* Method Description:
	*   This method returns the number of movies parsed from the XML file.
	**************************************************************************/
	public int getNumberOfMovies()
	{
		return m_MovieNL.getLength();
	}
	
	/**************************************************************************
	* Method Name:
	*   setMovie
	* Arguments Passed:
	*   int index = The index of the movie to set as current.
	* Data Returned:
	*   boolean = return true if index is valid, false otherwise
	* Method Description:
	*   This method sets the current movie to movie the index refrences.  This
	*   method also parses out the quotes and stores them into the m_Quotes
	*   array of Element objects.
	**************************************************************************/
	public boolean setMovie(int index)
	{
		if(index >= 0 && index < getNumberOfMovies())
		{
			m_MovieIndex = index;
			m_QuoteNL = ((Element)m_MovieNL.item(index)).getElementsByTagName("quote");
			
			return true;
		}
		
		return false;
	}
	
	/**************************************************************************
	* Method Name:
	*   getMovieIndex
	* Arguments Passed:
	*   None
	* Data Returned:
	*   int = Returns the index of the current movie. -1 means no movie has
	*     been set
	* Method Description:
	*   This method returns the index of the movie that has been previously
	*   set.
	**************************************************************************/
	public int getMovieIndex()
	{
		return m_MovieIndex;
	}
	
	/**************************************************************************
	* Method Name:
	*   setMovie
	* Arguments Passed:
	*   String movieTitle = The title of the movie to set as current.
	* Data Returned:
	*   boolean = return true if title is valid, false otherwise
	* Method Description:
	*   This method sets the current movie to passed string.  If found, it
	*   calls the setMovie(int index) method to set the current movie.
	**************************************************************************/
	public boolean setMovie(String movieTitle)
	{
		String movieTitles[] = getMovieTitles();
		
		for(int i = 0; i < movieTitles.length; i ++)
		{
			if(movieTitle.compareToIgnoreCase(movieTitles[i]) == 0)
			{
				return setMovie(i);
			}
		}
		
		return false;
	}
	
	/**************************************************************************
	* Method Name:
	*   getNumberOfQuotes
	* Arguments Passed:
	*   none
	* Data Returned:
	*   int = the number of quotes for the current movie
	* Method Description:
	*   This method returns the number of quotes that can be parsed for the
	*   current movie.
	**************************************************************************/
	public int getNumberOfQuotes()
	{
		return m_QuoteNL.getLength();
	}
	
	/**************************************************************************
	* Method Name:
	*   getQuote
	* Arguments Passed:
	*   int index = The index of the quote to return
	* Data Returned:
	*   String = If found, a quote at the passed index, null if not available
	* Method Description:
	*   This method returns the quote at the passed index, if available.
	**************************************************************************/
	public String getQuote(int index)
	{
		if(index >= 0 && index < getNumberOfQuotes())
		{
			return (m_QuoteNL.item(index).getTextContent()).trim();
		}
		
		return null;
	}
	
	/**************************************************************************
	* Method Name:
	*   getQuoteList
	* Arguments Passed:
	*   None
	* Data Returned:
	*   String[] = Returns a list of all the quotes in the set movie
	* Method Description:
	*   This method returns a list of all the quotes for the movie that has 
	*   been set
	**************************************************************************/
	public String[] getQuoteList()
	{
		String quotes[] = new String[getNumberOfQuotes()];
		
		for(int i = 0; i < getNumberOfQuotes(); i ++)
		{
			quotes[i] = (m_QuoteNL.item(i).getTextContent()).trim();
		}
		
		return quotes;
	}
}