/******************************************************************************
* File name:
*   MovieQuotes.java
* Author:
*   Barham, Ray
*
* Creation Date:
*   3/25/2007
* 
* Developed on:
*   WinXP
* Developed with:
*   Java(TM) SE Development Kit 6
*   JCreator 4.00.028
*
* Description:
*   This program display's quotes from a movie that the user selects.
*   The information must be stored in an XML file.  Look in 
*   MoviewQuoteParser.java file to see the required format of the XML file.   
******************************************************************************/

// Import statements
import java.io.*;

/******************************************************************************
 * Class Name:
 *   MovieQuotes
 * Base Class:
 *   None
 * Class Description:
 *   This is the class that contains main.  All the methods are static so there
 *   is no need to create an object of MovieQuotes.
 ******************************************************************************/
public class MovieQuotes
{
	// Error messages for making bad choices...
	// (and a bad attempt at being funny)
	private static String m_ErrorMsgs[] = {
		"\nWoops, that wasen't a choice.  Give it another shot.\n",
		"\nThat wasen't a choice, either.  Come on, it's not that hard.  Try again.\n",
		"\nYour trying to break me, right?  Sorry, not going to happen.\n"+
			"I was coded to well.</flex>  Now pick something.\n",
		"\nOk, that's it.  Now I'm mad.</roar>  Time to delete all your documents!\n"+
			"Now pick a valid choice before I get even more pissed off.\n",
		"\nIt's going to be like that, huh?  Fine, there goes all your music.\n"+
			"Now, pick something damnit!\n",
		"\nGRRR!  Say goodbye to all your pr0n!  See...see what you made me do...\n",
		"\n(Program wakes up with blood all over it's....code) Wow, what happened?\n"+
			"Who's blood is this?  Damn, I must have blacked out and killed someone again.\n"+
			"O'Well...they should have just picked a valid choice...\n"};
	
	// Message for when the program is exited
	private static String m_QuitMsg = 
		"\nFine, be a loser and quit.  I didn't like you anyway.</cry>\n"+
		"Just kidding...thanks for playing! :)\n";
	
	/**************************************************************************
	* Method Name:
	*   main
	* Arguments Passed:
	*   String[] args = The arguments being passed to the program.  This
	*     program only takes one argument, the XML file containing the movie
	*     and quote information.  (args[0] = filename.xml)
	* Data Returned:
	*   None
	* Method Description:
	*   This is the starting point of the program.  This method creates the
	*   File object for the XML file, passes that object to the 
	*   MovieQuoteParser object, and accesses the MovieQuoteParser object for 
	*   the parsed XML information
	**************************************************************************/
	public static void main(String[] args)
	{
		// Check to see if the XML file was passed
		if(args.length != 1)
		{
			System.out.println("Woops, no XML file passed.  Did you read the readme.rtf?");
			System.exit(1);
		}
		
		// Check to see if the file passed has a .xml extension
		else if(!args[0].endsWith(".xml"))
		{
			System.out.println("Reading readme.rtf files are fun!");
			System.out.println("They tell you things like \"the file passed needs to have a .xml extension\".");
			System.exit(1);
		}
		
		File XMLFile = new File(args[0]);
		
		// Check to see if the file is valid
		if(!XMLFile.isFile())
		{
			System.out.println("Sorry, but, that's not a file.  Checkout the readme.rtf for more info.");
			System.exit(1);
		}
		
		try
		{
			MovieQuoteParser mqp = new MovieQuoteParser(XMLFile);
			String titles[] = mqp.getMovieTitles();
			
			// If no movies are available...
			if(titles == null)
			{
				System.out.println("Dang, there's no movie's available.  That kinda puts a kink in the fun, huh?\n");
				System.exit(0);
			}
			
			int movieChoice;
			
			do
			{
				movieChoice = movieMenu(titles);
				
				// Done to coincide with the indexes in the MoviewQuoteParser
				movieChoice --;
				
				mqp.setMovie(movieChoice);
				
				// If the user wants to view quotes for a movie
				if(movieChoice != mqp.getNumberOfMovies())
				{
					String quotes[] = mqp.getQuoteList();
					
					System.out.println("\nThere are "+mqp.getNumberOfQuotes()+" quotes for "+titles[mqp.getMovieIndex()]+".");
					System.out.println("Hope you enjoy them.\n");
					System.out.println("Press the ENTER key to continue.");
					ConsoleInput.readString();
					
					viewQuotes(quotes);
				}
			}while(movieChoice != mqp.getNumberOfMovies());  // If the user wants to quit
			
			System.out.println(m_QuitMsg);
			System.exit(0);
		}
		
		catch (IOException e)
		{
			System.out.println(e);
		}
		
		catch (javax.xml.parsers.ParserConfigurationException e)
		{
			System.out.println(e);
		}
		
		catch (org.xml.sax.SAXException e)
		{
			System.out.println(e);
		}
		
		System.exit(1);
	}
	
	/**************************************************************************
	* Method Name:
	*   movieMenu
	* Arguments Passed:
	*   String[] movieList = An array of all the movie titles available.
	* Data Returned:
	*   None
	* Method Description:
	*   This method displays a menu of movies titles and returns the users 
	*   choice.
	**************************************************************************/
	public static int movieMenu(String[] movieList)
	{
		int movieCounter = 0;
		int errorCounter = 0;
		int movieChoice = 0;
		
		// Return if there are no movies
		if(movieList.length == 0)
			return -1;
		
		// Loop until a vaild choice has been given.
		do
		{
			System.out.println("\nSelect a movie to view quotes from:");
			
			// Display the movie titles
			for(int i = 0; i < movieList.length; i++)
			{
				System.out.println((i+1) + ") " + movieList[i]);
			}
			
			System.out.println((movieList.length+1)+") Quit");
			
			try
			{
				movieChoice = ConsoleInput.readInt();
				
				// Check to see if the number was valid.  If not vaild, display
				// the next funny error message
				if(movieChoice <= 0 || movieChoice > movieList.length+1)
				{
					System.out.println(m_ErrorMsgs[errorCounter]);
					errorCounter ++;
					
					if(errorCounter == m_ErrorMsgs.length)
						errorCounter = 0;
					
					movieCounter = 0;
				}
			}
			
			// Catches the NumberFormatException if the user entered
			// a non-number.  If the exception was caught, display
			// the next funny error message
			catch (NumberFormatException e)
			{
				System.out.println(m_ErrorMsgs[errorCounter]);
				errorCounter ++;
				
				if(errorCounter == m_ErrorMsgs.length)
					errorCounter = 0;
				
				movieCounter = 0;
			}
		}while(movieChoice <= 0 || movieChoice > movieList.length+1);
		
		return movieChoice;
	}
	
	/**************************************************************************
	* Method Name:
	*   viewQuotes
	* Arguments Passed:
	*   String quotes[] = An array of all the quotes for the specified movie
	* Data Returned:
	*   None
	* Method Description:
	*   This method displays the quotes from the specified movie.  The quotes
	*   will repeat until the user decides to quit.  It will notify the user
	*   when the quotes begin to repeat.
	**************************************************************************/
	public static void viewQuotes(String quotes[])
	{
		String quit = " ";
		int cnt = 0;
		boolean cont = true;
		
		// Loop until the user requests to quit
		while(cont)
		{
			// Display the quote #, the quote, and request user to specify
			// the next course of action
			System.out.println("=== Quote "+(cnt+1)+" ===");
			System.out.println(quotes[cnt]);
			System.out.println("\nPress the ENTER key to view next quote or type 'q' and press ENTER to quit.");
			
			quit = ConsoleInput.readString();
			
			if(quit.startsWith("q"))
				cont = false;
			else
			{
				cnt ++;
				if(cnt >= quotes.length)
				{
					System.out.println("Repeating quotes...\n");
					cnt = 0;
				}
			}
		}
	}
}